--     Island Explorer - An AIO island expeditions assistant
--     Copyright (C) 2020  Nivix
-- 
--     This program is free software: you can redistribute it and/or modify
--     it under the terms of the GNU General Public License as published by
--     the Free Software Foundation, either version 3 of the License, or
--     (at your option) any later version.
-- 
--     This program is distributed in the hope that it will be useful,
--     but WITHOUT ANY WARRANTY; without even the implied warranty of
--     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--     GNU General Public License for more details.
-- 
--     You should have received a copy of the GNU General Public License
--     along with this program.  If not, see <https://www.gnu.org/licenses/>.

CalendarBuddy = { };

CalendarBuddy.frame = CreateFrame("Frame");
CalendarBuddy.events = { };
CalendarBuddy.options = 
{
    ["sorting"] = 
    { 
        ["default"] = true,
        ["heroic"] = true,
        ["heroic_avg"] = true,
        ["mythic"] = true,
        ["mythic_avg"] = true
    },
    ["timeframe"] =
    { 
        ["today"] = true,
        ["historical"] = true
    },
    ["role"] =
    { 
        ["all"] = true,
        ["hps"] = true,
        ["dps"] = true,
        ["tank"] = true
    },
    ["zone"] =
    { 
        ["recent"] = "recent",
        ["uldir"] = 19,
        ["bod"] = 21,
        ["cos"] = 22,
        ["ep"] = 23,
        ["nya"] = 24
    } 
};

local CalBud = CalendarBuddy;
local events = CalBud.events;

SLASH_CALBUD1 = "/calbud"

SlashCmdList["CALBUD"] = function(msg, editbox)
    local parameter, value = msg:match("^(%S*)%s*(.-)$");
    if msg == "" then
        CalBudCopyFrame:Show()
        CalBudCopyFrameScroll:Show()
        CalBudCopyFrameScrollText:Show()
        CalBudCopyFrameScrollText:SetText(CalBud:eventString())
        CalBudCopyFrameScrollText:HighlightText()
        CalBudCopyFrameScrollText:SetScript("OnEscapePressed", function(self)
            CalBudCopyFrame:Hide()
        end)
        CalBudCopyFrameButton:SetScript("OnClick", function(self)
        CalBudCopyFrame:Hide()
        end)
        return
    end
    local parameter, value = msg:match("^(%S*)%s*(.-)$");
    if parameter == "reset" then
        CalBud.vars.sorting = "default";
        CalBud.vars.timeframe = "historical";
        CalBud.vars.role = "all";
        CalBud.vars.zone = "recent";
        print("CalBud: default values have been reset.");
    elseif CalBud.options[parameter] ~= nil and CalBud.options[parameter][value] ~= nil then
        CalBud.vars[parameter] = value;
        print("CalBud: default " .. parameter .. " is now set to " .. value .. ".");
    else 
        print("Calendar Buddy command list:\n/calbud\n/calbud reset\n/calbud sorting <default, heroic, heroic_avg, mythic, mythic_avg>\n/calbud timeframe <historical, today>\n/calbud role <all, dps, hps, tank>\n/calbud zone <recent, nya, ep, cos, bod, uldir>");
    end
end

function CalBud:event()
    local eventInfo = C_Calendar.GetEventInfo();
    if eventInfo == nil then
        return nil;
    end
    local event = { };
    event.info = eventInfo;
    event.count = C_Calendar.GetNumInvites();
    local inviteList = { };
    for idx = 1, event.count do
        inviteList[idx] = C_Calendar.EventGetInvite(idx);
    end
    event.invites = inviteList;
    return event;
end

function CalBud:infoToString(infoTable)
    local time = infoTable.time;
    local weekDay = CALENDAR_WEEKDAY_NAMES[time.weekday];
    local month = CALENDAR_FULLDATE_MONTH_NAMES[time.month];
    local dateString = string.format("%02d:%02d, %s, %d %s %d", time.hour, time.minute, weekDay, time.monthDay, month, time.year);
    return infoTable.title .. " - " .. dateString;
end

function CalBud:inviteToString(invite)
    local inviteString = "";
    local name = invite.name
    if name:find("-") == nil then
        name = name .. "-" .. CalBud.myRealm;
    end
    return name .. " " .. invite.inviteStatus;
end

function CalBud:invitesToString(invites, numInvites)
    local invitesString = CalBud:inviteToString(invites[1]);
    for idx = 2, numInvites do
        invitesString = invitesString .. ", " ..  CalBud:inviteToString(invites[idx], myRealm);
    end
    return invitesString;
end

function CalBud:argumentsString()
    local vars = CalBud.vars;
    local zone = CalBud.options.zone[vars.zone];
   return "sorting=" .. vars.sorting .. " timeframe=" .. vars.timeframe .. " zone=" .. zone .. " role=" .. vars.role;
end

function CalBud:eventString()
    local event = CalBud:event();
    if event == nil then
        return "Please open an event first";
    end
    return CalBud:infoToString(event.info) .. "\n" ..  CalBud:invitesToString(event.invites, event.count) .. "\n" .. CalBud:argumentsString();
end

function CalBud:init()
    if CalendarBuddySavedVariables == nil then
        CalendarBuddySavedVariables = { };
    end
    local SavedVars = CalendarBuddySavedVariables;
    if SavedVars.sorting == nil then
        SavedVars.sorting = "default";
    end
    if SavedVars.timeframe == nil then
        SavedVars.timeframe = "historical";
    end
    if SavedVars.zone == nil then
        SavedVars.zone = "recent";
    end
    if SavedVars.role == nil then
        SavedVars.role = "all";
    end
    CalBud.vars = SavedVars;
    CalBud.myRealm = GetRealmName();
end

function events:ADDON_LOADED(addonName)
    if addonName == "CalendarBuddy" then
        CalBud:init();
    end 
end      

function events:handler(event, ...)
	events[event](self, ...);
end

CalBud.frame:SetScript("OnEvent", events.handler); 
CalBud.frame:RegisterEvent("ADDON_LOADED");
